///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Filename:		Main.asm
//							(c) ANALOG DEVICES 2001
//
// Date:				12/12/2001
// Purpose:			Software Overlay Main Program
//
// Inputs:			none
// Outputs:			none
//
// Description:	This progam calls overlay functions which reside in external overlay memory. These external functions
//							are automatically loaded into the DSP's internal memory via the overlay manager and the DMA engine of the
//							ADSP-2191. After they are loaded into internal memory, they are then executed as a normal function.
//							
// Registers Used:
//
//							primary register set:		ax0, ax1, ay0, ay1
//							primary DAGs:						none
//							secondary register set:	none
//							secondary DAGs:					none
//
// Additional Information:
//
//							Memory size - 67 PM locations used
//
// Revision Information:
//
//							rev 0.0 - (12/12/01)
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


#include "2191asm.h"

.extern func1_add;											// external overlay functions called from this main program
.extern func2_mult;											// external overlay functions called from this main program
.extern func3_sub;											// external overlay functions called from this main program

.extern save_ax0;												// temporary memory locations for saving-restoring of ax0 register
.extern save_ay0;												// temporary memory locations for saving-restoring of ay0 register

.extern ovly_type;											// variable used to define the type of overlay to be loaded by overlay manager (=1 for PM)

.section/pm	seg_code;
.global start;

start:

	nop;
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	// Define this conditional definition if you need to clear all of the registers before running
	// this program. This is only for debugging purposes only and is not entirely neccessary.
	/////////////////////////////////////////////////////////////////////////////////////////////////

	#ifdef DEBUG
	
	// just clear all of the registers for debugging purposes and for a clean environment !!!!!
	ar = 0x0000;
	
	// clear ALU registers
	ax0 = ar; ay0 = ar; ax1 = ar; ay1 = ar;
	
	// clear DAG registers
	i0 = ar; i1 = ar; i2 = ar; i3 = ar; i4 = ar; i5 = ar; i6 = ar; i7 = ar;
	m0 = ar; m1 = ar; m2 = ar; m3 = ar; m4 = ar; m5 = ar; m6 = ar; m7 = ar;
	l0 = ar; l1 = ar; l2 = ar; l3 = ar; l4 = ar; l5 = ar; l6 = ar; l7 = ar;
	reg(b0) = ar; reg(b1) = ar; reg(b2) = ar; reg(b3) = ar; reg(b4) = ar; reg(b5) = ar; reg(b6) = ar; reg(b7) = ar; 
	
	// clear MAC registers
	mx0 = ar; mx1 = ar; my0 = ar; my1 = ar;
	mr0 = ar; mr1 = ar; mr2 = ar;
	
	// clear shifter registers
	si = ar; sb = ar; se = ar;
	sr0 = ar; sr1 = ar; sr2 = ar;
	
	#endif DEBUG

	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	// Assign all of the peripheral interrupt priorities (EXCEPT for MEM DMA) to the lowest
	// priority setting. MEM DMA will be assigned the highest interrupt priority level. This is
	// strictly for academic purposes since in this example, the only active interrupt will be
	// MEM DMA.
	/////////////////////////////////////////////////////////////////////////////////////////////////

Interrupt_Priority_Config:

	iopg = Clock_and_System_Control_Page;				// switch to iopg 0 for system configuration register
	ar = io(SYSCR);															// read value of system configuration register into ar
	ar = setbit 4 of ar;												// execute from on-chip SRAM starting at address 0x000000 (PM)

	#ifdef BUS_WIDTH_8
	ar = setbit 5 of ar;												// enable PF15:8 (8-bit external memory bus)
	#endif BUS_WIDTH_8
	
	io(SYSCR) = ar;															// write these values back into the system configuration register

	dis int;																		// globally disable all interrupts (we'll enable interrupts in the overlay manager for MEM DMA)
	imask = 0;																	// mask all interrupts

	iopg = Interrupt_Controller_Page;						// switch to the interrupt controller page (0x01)
	ax0 = 0xbbbb;																// set ax0 to lowest priority interrupt level
	io(IPR0) = ax0;															// assign lowest priority to host, SPORT0 rx/tx, & SPORT1 rx interrupts
	io(IPR1) = ax0;															// assign lowest priority to SPORT1 tx, SPORT2 rx/tx, & UART rx interrupts
	io(IPR2) = ax0;															// assign lowest priority to UART tx, timer 0-2 interrupts
	ax0 = 0xb0bb;
	io(IPR3) = ax0;															// assign highest priority to MEM DMA, lowest priority to FLAG A & B interrupts
	
	icntl = 0;																	// disable interrupt nesting
	irptl = 0;																	// clear all (pending) interrupts
	imask = 0x0010;															// allow only priority #4 interrupt (we've assigner #4 to MEM DMA)

	#ifdef DEBUG
	nop;																				// NOP added here for debug purposes only
	#endif DEBUG
	
	iopg = External_Memory_Interface_Page;			// switch to the EMI interface page (0x06)
	
	#ifdef BUS_WIDTH_8
	ax0 = 0x0030;																// configure EMI for 8-bit ext bus, ~RD/~WR acive low, DMA split disabled
	#else
	ax0 = 0x0038;																// configure EMI for 16-bit ext bus, ~RD/~WR acive low, DMA split disabled
	#endif BUS_WIDTH_8
	
	io(EMICTL) = ax0;

	/////////////////////////////////////////////////////////////////////////////////////////////////
	// Assign all of the mem space ctrl regs to zero wait states, internal wait mode only,
	// 1x core clock base clock div, and no wait hold cycle
	/////////////////////////////////////////////////////////////////////////////////////////////////

	ax0 = 0x0040;
	io(MS0CTL) = ax0;											// Memory Space Bank 0 Access Control Register
	io(MS1CTL) =ax0;                      // Memory Space Bank 1 Access Control Register
	io(MS2CTL) =ax0;                      // Memory Space Bank 2 Access Control Register
	io(MS3CTL) =ax0;                      // Memory Space Bank 3 Access Control Register
	io(IOMSCTL) = ax0;                    // IO Space Access Control Register
	
	iopg = External_Access_Bridge_Page; 	// set to IOPG 0x00
	ax0 = 0x0008;
	io(E_STAT) = ax0;											// switch to 24-bit data accesses =(0x0008): 16-bit = 0x0000



	/////////////////////////////////////////////////////////////////////////////////////////////////
	// Begin main program code below...
	/////////////////////////////////////////////////////////////////////////////////////////////////

Start_Program:

	ax1 = 5;
	ay1 = 3;

	call func1_add;												// call (and load) the first overlay function
	
	// return from overlay function call here...
	nop; nop; nop; nop; nop;

	ax0 = dm(save_ax0);										// restore ax0 register after returning from overlay manager
	ay0 = dm(save_ay0);										// restore ay0 register after returning from overlay manager

	call func1_add;												// call the first overlay function again

	// return from overlay function call here...
	nop; nop; nop; nop; nop;

	dm(save_ax0) = ax0;										// save ax0 register before calling overlay manager (which uses ax0)
	dm(save_ay0) = ay0;										// save ay0 register before calling overlay manager (which uses ay0)

	call func2_mult;											// call (and load) the second overlay function

	// return from overlay function call here...
	ax0 = dm(save_ax0);										// restore ax0 register after returning from overlay manager
	ay0 = dm(save_ay0);										// restore ay0 register after returning from overlay manager

	nop;																	// NOP added here for no particular reason

	dm(save_ax0) = ax0;										// save ax0 register before calling overlay manager (which uses ax0)
	dm(save_ay0) = ay0;										// save ay0 register before calling overlay manager (which uses ay0)

	call func3_sub;												// call (and load) the third overlay function

	// return from overlay function call here...
	ax0 = dm(save_ax0);										// restore ax0 register after returning from overlay manager
	ay0 = dm(save_ay0);										// restore ay0 register after returning from overlay manager


	nop;																	// NOPs added here for no particular reason
	nop;
	nop;
	nop;
	nop;
					
end:	

	nop;																	// NOPs added here for no particular reason
	nop;
	nop;
	nop;
	nop;
	
	jump end;
